# -*- coding: utf-8 -*-
from __future__ import division

# for WGS, how many reads needed to cover genome 1x
READS_PER_1X = 650000000 / 0.75 / 30

SEX_MAP = {'F': 'female', 'M': 'male', 'Unknown': 'unknown', 'unknown': 'unknown'}
